#include "StdAfx.h"
#include "DataContext.h"

#include "Binding.h"
//////////////////////////////////////////////////////////////////////////
//IDataContext
//////////////////////////////////////////////////////////////////////////
IDataContext* IDataContext::FindDataContext(LPCTSTR contextName)
{
	IDataContext* dataContext = NULL;
	int start = -1;
	CString rootDataContextName = GetToken(contextName, start);
	dataContext = CScriptletContainerFactory::get_instance()->FindDataContext(rootDataContextName);
	
	if (dataContext != NULL && start >= 0)
	{
		CString stringKey = contextName;
		stringKey = stringKey.Right(stringKey.GetLength() - start-1) + _T(".x");
		dataContext = dataContext->GetChildDataContext(stringKey);
	}
	return dataContext;
}
CString IDataContext::GetToken(LPCTSTR key, int& start)
{
	start += 1;
	CString subString;
	int lenKey = _tcslen(key);
	if (0 > start || start >= lenKey)
	{
		start = -1;
		return subString;
	}
	TCHAR ch = key[start];
	while (ch != _T('.'))
	{
		if (ch == _T('('))
		{
			start += 1;
			ch = key[start];
			while (ch != _T(')'))
			{
				subString.AppendChar(ch);
				start +=1;
				ch = key[start];
				if (start >= lenKey) break;
			}
			start +=1;
			break;
		}
		subString.AppendChar(ch);
		start +=1;
		ch = key[start];
		if (start >= lenKey) break;
	}
	if (start >= lenKey)
	{
		start = -1;
	}
	return subString;
}
IDataContext* IDataContext::GetChildDataContext(CString& propertyKey)
{
	IDataContext* childDataContext = this;

	int start = -1;
	CString element = GetToken(propertyKey, start);
	IDataContext* child = NULL;
	while (element.GetLength() > 0)
	{
		CAtlMap<CString, CComVariant>::CPair* pair = this->m_mapData.Lookup(element);
		if (pair == NULL) break;

		if (start == -1)
		{
			propertyKey = element;
			return childDataContext;			
		}
		else if (pair->m_value.vt == VT_UI4)
		{
			child = reinterpret_cast<IDataContext*>(UIntToPtr(pair->m_value.uintVal));
			if (child != NULL)
			{
				childDataContext = child;
				CString stringKey = propertyKey;
				propertyKey = stringKey.Right(stringKey.GetLength() - start-1);
				return child->GetChildDataContext(propertyKey);
			}
		}
		element = GetToken(element, start);
	}
	return childDataContext;
}
int IDataContext::GetCount()
{
	return m_mapData.GetCount(); 
}

IDataContext* IDataContext::GetChild(int index)
{
	CString name;
	name.Format(_T("%d"), index);	
	return GetChild(name);
}
IDataContext* IDataContext::GetChild(LPCTSTR name)
{
	CComVariant child = GetProperty(name);
	if (child.vt == VT_UI4)
	{
		return reinterpret_cast<IDataContext*>(UIntToPtr(child.uintVal));
	}
	return NULL;
}
void IDataContext::Add(LPCTSTR childName, IDataContext* child)
{
	CString name = childName;
	if (name.GetLength() == 0)
	{
		name.Format(_T("%d"), GetCount());
	}
	SetProperty(name, CComVariant(PtrToUint(child)));
}
void IDataContext::Remove(LPCTSTR name)
{
	SetProperty(name, CComVariant());
}

void IDataContext::Remove(int indexRemove)
{
	//CString name;
	//name.Format(_T("%d"), indexRemove);
	//Remove(name);
	ATLASSERT(FALSE); //̷ ߰ ϸ NULL Ѵ. "count" ״̱ 
//	m_mapData.
}
void IDataContext::RemoveAll()
{
	/*POSITION pos = m_mapData.GetStartPosition();
	while (pos)
	{
		CAtlMap<CString, CComVariant>::CPair* pair = m_mapData.GetNext(pos);
		if (pair)
		{
			if (pair->m_value.vt == VT_UI4)
			{
				IDataContext* childDataContext = reinterpret_cast<IDataContext*>(UIntToPtr(pair->m_value.uintVal));
				if (childDataContext != NULL)
				{
					childDataContext->RemoveAll();
				}
			}
		}
	}*/
	m_mapData.RemoveAll();
}
CComVariant IDataContext::GetSystemProperty(LPCTSTR propertyKey)
{
	if (_tcsicmp(propertyKey, _T("count")) == 0)
	{
		return CComVariant(GetCount());
	}
	return CComVariant();
}
//////////////////////////////////////////////////////////////////////////
// AbstractDataContext
//////////////////////////////////////////////////////////////////////////
AbstractDataContext::AbstractDataContext()
{
	m_PropertyBinder = new PropertyBinder();
}
AbstractDataContext::~AbstractDataContext()
{
	delete m_PropertyBinder;
	m_PropertyBinder = NULL;
}
PropertyBinder* AbstractDataContext::GetBinder()
{
	return m_PropertyBinder;
}
void AbstractDataContext::Sync()
{
	this->FirePropertyBindingListener(NULL);
}
void AbstractDataContext::FirePropertyBindingListener(LPCTSTR propertyKey)
{
	m_PropertyBinder->FirePropertyBindingListener(propertyKey, this);
}
//////////////////////////////////////////////////////////////////////////
// SimpleMapDataContext
//////////////////////////////////////////////////////////////////////////
CComVariant SimpleMapDataContext::GetProperty(LPCTSTR propertyKey)
{
	int start = -1;
	CString element = GetToken(propertyKey, start);
	SimpleMapDataContext* child = NULL;
	while (element.GetLength() > 0)
	{
		CAtlMap<CString, CComVariant>::CPair* pair = m_mapData.Lookup(element);
		if (pair == NULL) break;

		if (start == -1)
		{
			return pair->m_value;			
		}
		else if (pair->m_value.vt == VT_UI4)
		{
			child = reinterpret_cast<SimpleMapDataContext*>(UIntToPtr(pair->m_value.uintVal));
			if (child != NULL)
			{
				CString stringKey = propertyKey;
				CString subKey = stringKey.Right(stringKey.GetLength() - start-1);
				return child->GetProperty(subKey);
			}
		}
		element = GetToken(element, start);
	}
	return GetSystemProperty(propertyKey);
}
void SimpleMapDataContext::SetProperty(LPCTSTR propertyKey, CComVariant propertyValue)
{
	CString childKey = propertyKey;
	IDataContext* childDataContext = GetChildDataContext(childKey);
	if (childDataContext != NULL && childDataContext != this)
	{
		childDataContext->SetProperty(childKey, propertyValue);
		return;
	}

	CString key(propertyKey);
	CAtlMap<CString, CComVariant>::CPair* pair = m_mapData.Lookup(key);
	if (pair != NULL)
	{
		m_mapData.RemoveKey(propertyKey);
	}
	if (propertyValue.vt != VT_EMPTY)
	{
		m_mapData.SetAt(key, propertyValue);
	}
	FirePropertyBindingListener(propertyKey);
}
